from fastapi import APIRouter, Depends, HTTPException, Request
from sqlalchemy.orm import Session
from jose import jwt, ExpiredSignatureError, JWTError
from datetime import datetime, timedelta
from app.database import get_db
from app.core.security import create_access_token
from app.schemas.auth import LoginRequest
from app.models import member, auth
from app.models.token import UserToken
from app.models.member import Member
from app.models.msg import Msg
from app.core.deps import verify_token_and_get_current_user
from app.core.security import create_access_token, invalidate_token, verify_token
from app.crud.auth import create_login_failure_log
from app.core.security import verify_token
from app.core.security import SECRET_KEY, ALGORITHM
from slowapi import Limiter
from slowapi.util import get_remote_address

limiter = Limiter(key_func=get_remote_address)

router = APIRouter(prefix="/api/auth", tags=["authentication"])

ACCESS_TOKEN_EXPIRE_MINUTES = 120  # 토큰 만료 시간 설정

def check_login_attempts(db: Session, user_id: str) -> bool:
    count = db.query(auth.MemberNotLoginLog) \
        .filter(auth.MemberNotLoginLog.id == user_id,
                auth.MemberNotLoginLog.chk_admin == '0') \
        .count()
    return count >= 10

def check_recent_ip_login(db: Session, ip: str, user_id: str) -> bool:
    five_mins_ago = datetime.utcnow() - timedelta(minutes=5)
    return db.query(auth.MemberLoginLog) \
        .filter(auth.MemberLoginLog.ip == ip,
                auth.MemberLoginLog.id != user_id,
                auth.MemberLoginLog.regdate >= five_mins_ago) \
        .count() > 0

def delete_expired_tokens(db: Session):
    """
    만료된 토큰 삭제
    """
    current_time = datetime.utcnow()
    deleted_rows = db.query(UserToken).filter(UserToken.is_active == True, UserToken.created_at + timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES) < current_time).delete()
    db.commit()
    return deleted_rows

@router.post("/login")
@limiter.limit("1 per 3 seconds")
async def login(request: Request,
                login_data: LoginRequest,
                db: Session = Depends(get_db)):
    current_time = datetime.utcnow()
    expire_time = current_time + timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)
    user_ip = login_data.client_ip
    browser_info = login_data.browser_info

    # 1. 기본 입력값 체크
    if not login_data.username or not login_data.password:
        raise HTTPException(status_code=400, detail="로그인 정보를 확인하세요.")

    # 2. 로그인 시도 횟수 체크
    if check_login_attempts(db, login_data.username):
        create_login_failure_log(db, login_data.username, login_data.password, user_ip, "비밀번호 오류")
        raise HTTPException(status_code=400, detail="로그인 정보를 확인하세요.")

    # 3. IP 로그인 시도 체크
    if check_recent_ip_login(db, user_ip, login_data.username):
        create_login_failure_log(db, login_data.username, login_data.password, user_ip, "비밀번호 오류")
        raise HTTPException(status_code=400, detail="다른 계정으로 최근 로그인한 IP입니다.")

    # 4. 블록된 IP 체크
    blocked_ip = db.query(auth.BlockIpList) \
        .filter(auth.BlockIpList.ipaddr == user_ip,
                auth.BlockIpList.flag == 1) \
        .first()
    if blocked_ip:
        create_login_failure_log(db, login_data.username, login_data.password, user_ip, "아이피블럭")
        raise HTTPException(status_code=400, detail="현재 아이피로 로그인 할 수 없습니다.")

    # 5. 회원 정보 조회
    user = db.query(member.Member) \
        .filter(member.Member.id == login_data.username) \
        .first()

    if not user:
        create_login_failure_log(db, login_data.username, login_data.password, user_ip, "아이디 오류")
        raise HTTPException(status_code=400, detail="로그인 정보를 확인하세요.")

    # 6. 비밀번호 체크
    if user.pw != login_data.password:
        create_login_failure_log(db, login_data.username, login_data.password, user_ip, "비밀번호 오류")
        raise HTTPException(status_code=400, detail="로그인 정보를 확인하세요.")

    # 7. 계정 상태 체크
    if user.active != 1:
        create_login_failure_log(db, login_data.username, login_data.password, user_ip, "계정 상태 오류")
        raise HTTPException(status_code=400, detail="계정 상태를 확인하세요.")

    # 기존 토큰 비활성화
    db.query(UserToken).filter(UserToken.username == login_data.username, UserToken.is_active == True).update({"is_active": False})
    db.commit()

    # 새 토큰 발급
    access_token = create_access_token(username=login_data.username, ip=user_ip, db=db)

    # 로그인 로그 기록
    login_log = auth.MemberLoginLog(
        id=user.id,
        lvl=user.lvl,
        nick=user.nick,
        ip=user_ip,
        site=str(request.base_url),
        agent=browser_info,
        regdate=current_time
    )
    db.add(login_log)

    # 접속 정보 업데이트
    user.connect_code = user_ip
    user.connect_brow = browser_info
    user.actionTime = current_time
    db.commit()

    return {"access_token": access_token, "token_type": "bearer"}

@router.post("/verify")
@limiter.limit("1 per 1 second")
async def verify_token_route(
    request: Request,
    token: str,
    db: Session = Depends(get_db)
):
    """
    토큰 검증 및 갱신된 토큰 반환, 읽지 않은 메시지 개수 포함
    """
    # 토큰 검증 및 payload 추출
    payload = verify_token(token=token, db=db)

    # 새 만료 시간 계산
    current_time = datetime.utcnow()
    new_expire_time = current_time + timedelta(minutes=ACCESS_TOKEN_EXPIRE_MINUTES)

    # 토큰 갱신
    payload["exp"] = new_expire_time  # 새 만료 시간 갱신
    new_token = jwt.encode(payload, SECRET_KEY, algorithm=ALGORITHM)

    # 데이터베이스 갱신
    db.query(UserToken).filter(UserToken.token == token).update({
        "token": new_token,
        "created_at": current_time
    })
    db.commit()

    # 읽지 않은 메시지 개수 조회
    p_id = payload.get("sub")
    unread_count = db.query(Msg).filter(
        Msg.in_id == p_id,
        Msg.viewFlag == 0,
        Msg.del_chk == 0
    ).count()

    return {
        "status": "success",
        "new_token": new_token,  # 갱신된 토큰 반환
        "user": {
            "id": payload["sub"],
            "ip": payload["ip"],
            "exp": new_expire_time,
            "unread_messages": unread_count  # 읽지 않은 메시지 개수 포함
        }
    }

@router.post("/logout")
async def logout(
    request: Request,
    token: str,
    db: Session = Depends(get_db)
):
    """
    사용자가 전달한 토큰을 비활성화하여 로그아웃 처리.
    """
    try:
        payload = jwt.decode(token, SECRET_KEY, algorithms=[ALGORITHM])
        username = payload.get("sub")

        if username is None:
            raise HTTPException(status_code=400, detail="유효하지 않은 토큰입니다.")

        # 토큰 비활성화
        db.query(UserToken).filter(UserToken.token == token, UserToken.is_active == True).update({"is_active": False})
        db.commit()

        return {"status": "success", "message": "로그아웃이 성공적으로 처리되었습니다."}

    except ExpiredSignatureError:
        raise HTTPException(status_code=400, detail="이미 만료된 토큰입니다.")
    except JWTError:
        raise HTTPException(status_code=400, detail="토큰 검증에 실패했습니다.")

@router.post("/logout_all")
async def logout_all(
    username: str,
    db: Session = Depends(get_db)
):
    """
    특정 사용자의 모든 활성화된 토큰을 비활성화.
    """
    updated_rows = db.query(UserToken).filter(UserToken.username == username, UserToken.is_active == True).update({"is_active": False})
    db.commit()

    if updated_rows == 0:
        raise HTTPException(status_code=404, detail="활성화된 토큰이 없습니다.")

    return {"status": "success", "message": f"{username}의 모든 활성화된 토큰이 비활성화되었습니다."}

@router.get("/active_tokens")
async def get_active_tokens(
    db: Session = Depends(get_db)
):
    """
    활성화된 토큰의 사용자 이름과 토큰 반환
    """
    active_tokens = db.query(UserToken).filter(UserToken.is_active == True).all()
    return [
        {
            "username": token.username,
            "token": token.token
        }
        for token in active_tokens
    ]

@router.put("/change-password")
async def change_password(user_id: str, new_password: str, db: Session = Depends(get_db)):
    """
    비밀번호 변경 엔드포인트
    """
    if not user_id or not new_password:
        raise HTTPException(status_code=400, detail="유효하지 않은 요청입니다. 사용자 ID와 새로운 비밀번호는 필수입니다.")

    try:
        # 유저 정보 가져오기
        user = db.query(member.Member).filter(member.Member.id == user_id).first()
        if not user:
            raise HTTPException(status_code=404, detail="회원 정보를 찾을 수 없습니다.")

        # 유저 활성 상태 확인
        if user.active != 1:
            raise HTTPException(status_code=403, detail="로그 아웃 후 재신청 해주세요.")

        # 비밀번호 변경
        user.pw = new_password
        db.commit()

        return {"status": "success", "message": "비밀번호가 성공적으로 변경되었습니다."}

    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"비밀번호 변경 중 오류가 발생했습니다: {str(e)}")

