from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session
from app.database import get_db
from app.models.board_as import BoardAs
from app.models.member import Member
from app.models.msg import Msg
from datetime import datetime
from pytz import timezone

router = APIRouter(prefix="/api/board", tags=["Board"])


@router.get("/user-queries")
async def get_user_queries(user_id: str, db: Session = Depends(get_db)):
    """
    유저의 1:1 문의와 응답을 반환하는 API 엔드포인트
    """
    if not user_id:
        raise HTTPException(status_code=400, detail="유효하지 않은 사용자 ID입니다.")

    try:
        # BoardAs와 Msg를 조인, del_flag가 0인 데이터만 가져옴
        results = db.query(
            BoardAs.idx.label("question_id"),
            BoardAs.title.label("question_title"),
            BoardAs.contents.label("question_contents"),
            BoardAs.regdate.label("question_regdate"),
            BoardAs.admin_read.label("admin_read"),  # admin_read 추가
            Msg.contents.label("response_contents"),
            Msg.regdate.label("response_regdate")
        ).outerjoin(
            Msg, BoardAs.idx == Msg.as_idx
        ).filter(
            BoardAs.id == user_id,  # 해당 유저의 문의
            BoardAs.del_flag == 0  # 삭제되지 않은 문의
        ).order_by(BoardAs.regdate.desc()).all()

        if not results:
            return {"status": "success", "data": []}

        # 데이터를 변환하여 반환
        data = [
            {
                "question": {
                    "id": row.question_id,
                    "title": row.question_title,
                    "contents": row.question_contents,
                    "regdate": row.question_regdate.strftime("%Y-%m-%d %H:%M:%S") if row.question_regdate else None,
                    "admin_read": row.admin_read  # 관리자 읽음 상태 추가
                },
                "response": {
                    "contents": row.response_contents if row.response_contents else None,
                    "regdate": row.response_regdate.strftime("%Y-%m-%d %H:%M:%S") if row.response_regdate else None
                } if row.response_contents else None
            }
            for row in results
        ]

        return {"status": "success", "data": data}

    except Exception as e:
        raise HTTPException(status_code=500, detail=f"데이터를 가져오는 중 오류가 발생했습니다: {str(e)}")

@router.post("/register")
async def register_board_as(
    user_id: str,  # 사용자 ID
    title: str,  # 게시글 제목
    contents: str,  # 게시글 내용
    ip: str,  # IP 주소
    match_seqs: str = None,  # 매치 리스트 문자열, 선택적
    db: Session = Depends(get_db)
):
    """
    1:1 문의를 등록하는 API 엔드포인트
    """
    if not user_id:
        raise HTTPException(status_code=400, detail="유효하지 않은 사용자 ID입니다.")

    if not title or not contents:
        raise HTTPException(status_code=400, detail="제목과 내용은 필수 항목입니다.")

    if not ip:
        raise HTTPException(status_code=400, detail="IP 주소는 필수 항목입니다.")

    try:
        # 사용자 정보를 가져옴
        member = db.query(Member).filter(Member.id == user_id).first()
        if not member:
            raise HTTPException(status_code=404, detail="회원 정보를 찾을 수 없습니다.")

        # 활성화 상태 확인
        if member.active != 1:
            raise HTTPException(status_code=403, detail="로그아웃 후 다시 시도해 주세요.")

        # 기존 문의 확인 (admin_read가 0인 데이터 존재 여부 체크)
        existing_inquiry = db.query(BoardAs).filter(BoardAs.id == user_id, BoardAs.admin_read == 0).first()
        if existing_inquiry:
            return {"status": "error", "message": "기존 문의 확인 중입니다. 답변을 기다려 주세요."}

        # 닉네임 및 레벨 정보
        user_nick = member.nick
        user_lvl = member.lvl
        site_lvl = member.site_lvl

        # 등록 시간 (KST)
        kst = timezone("Asia/Seoul")
        regdate = datetime.now(kst)

        # board_as 데이터 삽입
        new_board_as = BoardAs(
            id=user_id,
            lvl=user_lvl,
            site_lvl=site_lvl,
            nick=user_nick,
            title=title,
            contents=contents,
            match_seqs=match_seqs,
            regdate=regdate,
            ip=ip,  # IP 주소 저장
            admin_read=0,  # 관리자 읽음 상태 초기값
            del_flag=0,  # 사용자 삭제 상태 초기값
            admin_del=0  # 관리자 삭제 상태 초기값
        )
        db.add(new_board_as)
        db.commit()

        return {"status": "success", "message": "1:1 문의가 성공적으로 등록되었습니다."}

    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"문의 등록 중 오류가 발생했습니다: {str(e)}")


@router.delete("/delete")
async def delete_board_as(
        user_id: str,  # 사용자 ID
        idx: int,  # 게시글 ID (board_as.idx)
        db: Session = Depends(get_db)
):
    """
    특정 1:1 문의를 삭제하는 API 엔드포인트
    """
    if not user_id or not idx:
        raise HTTPException(status_code=400, detail="유효하지 않은 요청입니다. 사용자 ID와 게시글 ID는 필수입니다.")

    try:
        # 사용자 ID와 idx에 해당하는 레코드를 찾음
        board_as_entry = db.query(BoardAs).filter(BoardAs.id == user_id, BoardAs.idx == idx).first()

        if not board_as_entry:
            raise HTTPException(status_code=404, detail="해당 게시글을 찾을 수 없습니다.")

        # 삭제 플래그를 설정
        board_as_entry.del_flag = 1
        db.commit()

        return {"status": "success", "message": "게시글이 성공적으로 삭제되었습니다."}

    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"게시글 삭제 중 오류가 발생했습니다: {str(e)}")