from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session
from datetime import datetime
from app.database import get_db
from app.models.board_free import BoardFree
from typing import List, Optional

router = APIRouter(prefix="/api/board-free", tags=["BoardFree"])

# 게시글 목록 조회
@router.get("/list")
async def get_board_free_list(db: Session = Depends(get_db)):
    """
    게시글 목록 조회 API
    """
    try:
        # 게시글 목록 가져오기
        posts = db.query(BoardFree.idx, BoardFree.title, BoardFree.regdate, BoardFree.nick, BoardFree.site_lvl).order_by(BoardFree.regdate.desc()).all()

        return {"status": "success", "data": posts}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"목록 조회 중 오류가 발생했습니다: {str(e)}")


# 게시글 보기
@router.get("/{idx}")
async def get_board_free_detail(idx: int, db: Session = Depends(get_db)):
    """
    게시글 상세 조회 API
    """
    try:
        # 게시글 가져오기
        post = db.query(BoardFree).filter(BoardFree.idx == idx).first()
        if not post:
            raise HTTPException(status_code=404, detail="게시글을 찾을 수 없습니다.")

        return {
            "status": "success",
            "data": {
                "idx": post.idx,
                "title": post.title,
                "regdate": post.regdate,
                "nick": post.nick,
                "site_lvl": post.site_lvl,
                "contents": post.contents,
                "match_seqs": post.match_seqs,
            },
        }
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"상세 조회 중 오류가 발생했습니다: {str(e)}")


# 게시글 작성
@router.post("/create")
async def create_board_free(
    title: str,
    contents: str,
    nick: str,
    site_lvl: int,
    match_seqs: Optional[str] = None,
    db: Session = Depends(get_db),
):
    """
    게시글 작성 API
    """
    if not all([title, contents, nick, site_lvl]):
        raise HTTPException(status_code=400, detail="필수 항목이 누락되었습니다.")

    try:
        # 등록 시간 설정
        regdate = datetime.utcnow()

        # 게시글 생성
        new_post = BoardFree(
            title=title,
            contents=contents,
            nick=nick,
            site_lvl=site_lvl,
            match_seqs=match_seqs,
            regdate=regdate,
        )
        db.add(new_post)
        db.commit()

        return {"status": "success", "message": "게시글이 성공적으로 작성되었습니다.", "idx": new_post.idx}
    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"게시글 작성 중 오류가 발생했습니다: {str(e)}")


# 게시글 수정
@router.put("/update/{idx}")
async def update_board_free(idx: int, contents: str, db: Session = Depends(get_db)):
    """
    게시글 수정 API
    """
    try:
        # 게시글 가져오기
        post = db.query(BoardFree).filter(BoardFree.idx == idx).first()
        if not post:
            raise HTTPException(status_code=404, detail="게시글을 찾을 수 없습니다.")

        # 내용 업데이트
        post.contents = contents
        db.commit()

        return {"status": "success", "message": "게시글이 성공적으로 수정되었습니다."}
    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"게시글 수정 중 오류가 발생했습니다: {str(e)}")
