from fastapi import APIRouter, Depends, HTTPException, Request
from sqlalchemy.orm import Session
from app.database import get_db
from app.core.security import verify_token
from app.models.msg import Msg
from datetime import datetime

router = APIRouter(prefix="/api/msg", tags=["messages"])

@router.get("/list")
async def get_messages(
    db: Session = Depends(get_db),
    user_id: str = None  # 클라이언트에서 요청 시 사용자 ID를 전달
):
    """
    사용자 ID로 메시지 목록 가져오기
    """
    if not user_id:
        raise HTTPException(status_code=400, detail="회원 정보가 없습니다.")

    # 메시지 목록 조회
    messages = db.query(Msg).filter(
        Msg.del_chk == 0,
        Msg.in_id == user_id
    ).order_by(Msg.regdate.desc()).all()

    # 메시지 데이터 변환
    data = []
    for message in messages:
        data.append({
            "idx": message.idx,
            "contents": message.contents.replace('\n', '<br>'),  # 줄바꿈을 <br>로 변환
            "strim": (message.contents[:40] + '...') if len(message.contents) > 40 else message.contents,
            "regdate": message.regdate.strftime("%Y-%m-%d %H:%M:%S") if message.regdate else None,
            "chk_date": message.chk_date.strftime("%Y-%m-%d %H:%M:%S") if message.chk_date else '&nbsp;',
            "viewFlag": message.viewFlag
        })

    return {"status": "success", "data": data}

@router.get("/unread_count")
async def get_unread_message_count(
    request: Request,
    token: str,
    db: Session = Depends(get_db)
):
    """
    읽지 않은 메시지 개수를 반환합니다.
    """
    # 토큰 검증
    payload = verify_token(token=token, db=db)
    p_id = payload.get("sub")
    if not p_id:
        raise HTTPException(status_code=401, detail="유효하지 않은 사용자입니다.")

    # 메시지 카운트 쿼리
    unread_count = db.query(Msg).filter(
        Msg.in_id == p_id,
        Msg.viewFlag == 0,
        Msg.del_chk == 0
    ).count()

    return {"unread_count": unread_count}

@router.post("/read")
async def read_message(
    message_id: int,  # 읽을 메시지의 ID
    user_id: str,     # 사용자 ID
    db: Session = Depends(get_db)
):
    """
    메시지 읽음 처리
    """
    # 메시지 조회
    message = db.query(Msg).filter(
        Msg.idx == message_id,
        Msg.in_id == user_id,
        Msg.del_chk == 0  # 삭제되지 않은 메시지만 처리
    ).first()

    if not message:
        raise HTTPException(status_code=404, detail="메시지를 찾을 수 없습니다.")

    if message.viewFlag == 1:
        return {"status": "success", "message": "이미 읽은 메시지입니다."}

    # 읽음 상태 업데이트
    message.viewFlag = 1
    db.commit()

    return {"status": "success", "message": "메시지를 읽음 처리했습니다."}


@router.post("/delete")
async def delete_message(
    message_id: int,  # 삭제할 메시지의 ID
    user_id: str,     # 사용자 ID
    db: Session = Depends(get_db)
):
    """
    메시지 삭제 플래그 설정
    """
    # 메시지 조회
    message = db.query(Msg).filter(
        Msg.idx == message_id,
        Msg.in_id == user_id,
        Msg.del_chk == 0  # 이미 삭제되지 않은 메시지만 처리
    ).first()

    if not message:
        raise HTTPException(status_code=404, detail="메시지를 찾을 수 없습니다.")

    # 삭제 플래그 설정
    message.del_chk = 1
    db.commit()

    return {"status": "success", "message": "메시지를 삭제 처리했습니다."}

@router.post("/read_all")
async def read_all_messages(
    user_id: str,  # 사용자 ID
    db: Session = Depends(get_db)
):
    """
    사용자의 모든 메시지를 읽음 처리
    """
    # 사용자의 읽지 않은 메시지 조회
    unread_messages = db.query(Msg).filter(
        Msg.in_id == user_id,
        Msg.viewFlag == 0,
        Msg.del_chk == 0
    ).all()

    if not unread_messages:
        return {"status": "success", "message": "읽지 않은 메시지가 없습니다."}

    # 모든 읽지 않은 메시지 읽음 처리
    for message in unread_messages:
        message.viewFlag = 1

    db.commit()

    return {"status": "success", "message": f"{len(unread_messages)}개의 메시지를 읽음 처리했습니다."}


@router.post("/delete_all")
async def delete_all_messages(
    user_id: str,  # 사용자 ID
    db: Session = Depends(get_db)
):
    """
    사용자의 모든 메시지를 삭제 처리
    """
    # 사용자의 삭제되지 않은 메시지 조회
    messages_to_delete = db.query(Msg).filter(
        Msg.in_id == user_id,
        Msg.del_chk == 0
    ).all()

    if not messages_to_delete:
        return {"status": "success", "message": "삭제할 메시지가 없습니다."}

    # 모든 메시지 삭제 처리
    for message in messages_to_delete:
        message.del_chk = 1

    db.commit()

    return {"status": "success", "message": f"{len(messages_to_delete)}개의 메시지를 삭제 처리했습니다."}
