from datetime import datetime
from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session
from sqlalchemy.exc import SQLAlchemyError
from app.database import get_db
from app.models.member import Member
from app.models.member_info import MembersInfo
from app.models.recommender_code import RecommenderCode
from app.models.site_config import GameConfig

router = APIRouter(prefix="/api/check", tags=["Availability"])

@router.get("/partner")
async def check_partner_code(
    join_partner: str = None,  # 추천인 코드
    db: Session = Depends(get_db)
):
    """
    추천인 코드 확인 API
    """
    try:
        # 기본 설정
        parent_lvl = 6
        end_lvl = 7

        # game_config에서 추천인 관련 설정 확인
        game_config = db.query(GameConfig).filter(GameConfig.set_id == "admin").first()
        if not game_config:
            raise HTTPException(status_code=500, detail="게임 설정을 찾을 수 없습니다.")

        if game_config.g1_use_join01 != "y" or game_config.g1_use_join02 != "y":
            return {"status": "error", "message": "추천인 가입이 불가능합니다.", "checkPartner": 0}

        # 추천인 코드 확인
        recommender_code = db.query(RecommenderCode).filter(
            RecommenderCode.join_code == join_partner,
            RecommenderCode.close == 0,
            RecommenderCode.limit_date > datetime.utcnow(),
            RecommenderCode.re_use == 0
        ).first()

        if recommender_code:
            # 추천인 코드가 유효한 경우
            return {
                "status": "success",
                "message": "추천인이 확인되었습니다.",
                "checkPartner": "y",
                "partner": join_partner
            }

        # 추천인 코드가 아닌 추천인 ID 확인
        partner_info = db.query(MembersInfo).filter(MembersInfo.mem_id == join_partner).first()
        if not partner_info or partner_info.partner_chk != "y":
            return {"status": "error", "message": "라이브챗 문의 바랍니다.", "checkPartner": 0}

        # 추천인이 유효한지 확인
        partner = db.query(Member).filter(
            Member.id == join_partner,
            Member.lvl.in_([parent_lvl, end_lvl]),
            Member.active.in_([1, 5, 6])
        ).first()

        if not partner:
            return {"status": "error", "message": "라이브챗 문의 바랍니다.", "checkPartner": 0}

        # 추천인 코드가 없는 경우의 처리
        if game_config.g1_use_join02 == "n" and not join_partner:
            return {
                "status": "success",
                "message": "추천인 없이 가입합니다.",
                "checkPartner": "y"
            }

        # 추천인 코드가 유효한 경우
        return {
            "status": "success",
            "message": "추천인이 확인되었습니다.",
            "checkPartner": "y",
            "partner": join_partner
        }

    except SQLAlchemyError as e:
        raise HTTPException(status_code=500, detail=f"데이터베이스 오류가 발생했습니다: {str(e)}")
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"알 수 없는 오류가 발생했습니다: {str(e)}")
