from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session
from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from pytz import timezone
from app.database import get_db
from app.models.member import Member
from app.models.request_money import RequestMoney
from app.models.total_money_log import TotalMoneyLog
from app.models.member_info import MembersInfo

router = APIRouter()

@router.post("/convert_points")
async def convert_points(
    user_id: str,
    site: str,
    db: Session = Depends(get_db)
):
    try:
        # Fetch member details
        member = db.query(Member).filter(Member.id == user_id).first()
        if not member:
            raise HTTPException(status_code=404, detail="회원 정보를 찾을 수 없습니다.")
        if member.active != 1:
            raise HTTPException(status_code=400, detail="로그 아웃 후 재신청 해주세요.")
        if member.point_money < 1:
            raise HTTPException(status_code=400, detail="전환 할 포인트가 없습니다.")

        # Check for pending or in-progress requests
        existing_request = db.query(RequestMoney).filter(
            RequestMoney.request_type == "입금",
            RequestMoney.to_id == user_id,
            RequestMoney.result.in_(["2", "3"])  # 2: Pending, 3: In-progress
        ).first()

        if existing_request:
            raise HTTPException(status_code=400, detail="처리 중인 충전 내역이 있습니다. 기존 요청이 완료된 후 다시 시도해 주세요.")

        # Fetch member_info details
        member_info = db.query(MembersInfo).filter(MembersInfo.mem_id == user_id).first()
        if not member_info:
            raise HTTPException(status_code=404, detail="회원 은행 정보를 찾을 수 없습니다.")

        # Deduct points
        request_point_money = member.point_money
        member.point_money = 0
        db.commit()

        # Record in request_money
        deposit_request = RequestMoney(
            request_type="입금",
            to_id=member.id,
            to_lvl=member.lvl,
            from_id="admin",  # Default admin
            from_lvl=2,  # Default admin level
            p_lvl_1=member.p_lvl_1,
            p_lvl_2=member.p_lvl_2,
            p_lvl_3=member.p_lvl_3,
            p_lvl_4=member.p_lvl_4,
            p_lvl_5=member.p_lvl_5,
            p_lvl_6=member.p_lvl_6,
            account_info="포인트전환",
            account_name=member.nick,
            prv_money=member.money,
            request_money=request_point_money,
            money_type="online",
            mem_type="member",
            phone=member_info.phone,
            ip_addr="127.0.0.1",  # 실제 IP로 대체 필요
            result="2",
            regdate=datetime.now(timezone("Asia/Seoul")),
            site=site
        )
        db.add(deposit_request)
        db.commit()

        # Update member's money and admin's money
        admin = db.query(Member).filter(Member.id == "admin").first()
        if not admin or admin.money < request_point_money:
            raise HTTPException(status_code=400, detail="관리자 보유머니가 부족합니다.")
        admin.money -= request_point_money
        member.money += request_point_money

        # Update request_money result
        deposit_request.result = "1"
        deposit_request.checkDate = datetime.now(timezone("Asia/Seoul"))
        db.commit()

        # Log transaction
        log_entry = TotalMoneyLog(
            id=member.id,
            lvl=member.lvl,
            nick=member.nick,
            p_lvl_1=member.p_lvl_1,
            p_lvl_2=member.p_lvl_2,
            p_lvl_3=member.p_lvl_3,
            p_lvl_4=member.p_lvl_4,
            p_lvl_5=member.p_lvl_5,
            p_lvl_6=member.p_lvl_6,
            money_type="포인트전환",
            request_type="money",
            request_money=request_point_money,
            prv_money=deposit_request.prv_money,
            next_money=member.money,
            prv_point_money=request_point_money,
            next_point_money=0,
            regdate=datetime.now(timezone("Asia/Seoul"))
        )
        db.add(log_entry)
        db.commit()

        return {"status": "success", "message": "포인트 전환이 완료되었습니다."}

    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"데이터베이스 오류가 발생했습니다: {str(e)}")
    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"알 수 없는 오류가 발생했습니다: {str(e)}")