from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session
from sqlalchemy.sql import text
from app.database import get_db
from datetime import datetime

router = APIRouter(prefix="/api/points", tags=["Points"])


@router.get("/logs")
async def get_point_logs(user_id: str, db: Session = Depends(get_db)):
    """
    포인트 로그를 반환하는 API 엔드포인트
    """
    if not user_id:
        raise HTTPException(status_code=400, detail="회원 정보가 없습니다.")

    # SQL 쿼리
    point_query = text("""
        SELECT source, idx, regdate, etc, 
               type, 
               amount, 
               prv_amount, 
               next_amount, 
               vendor, game
        FROM (
            SELECT 'total_money_log' AS source, idx, regdate, etc, money_type AS type, request_money AS amount, prv_point_money AS prv_amount, next_point_money AS next_amount, NULL AS vendor, NULL AS game
            FROM total_money_log
            WHERE (request_type = 'point' OR (request_type = 'money' AND money_type = '포인트전환')) 
            AND money_type NOT LIKE '%루징%' AND id = :user_id

            UNION ALL

            SELECT 'bet_reward_log' AS source, b.idx, b.regdate, b.etc, b.reward_type AS type, b.reward AS amount, b.prv_reward AS prv_amount, b.next_reward AS next_amount, c.vendor, c.game
            FROM bet_reward_log b
            LEFT JOIN casino_log c ON c.log_index = CONCAT(b.etc, '_b')
            WHERE b.id = :user_id
        ) AS combined_results
        ORDER BY regdate DESC
        LIMIT 100;
    """)

    try:
        # 쿼리 실행
        result = db.execute(point_query, {"user_id": user_id}).mappings().fetchall()

        # 데이터 변환
        data = []
        for row in result:
            type_cleaned = row["type"].replace("GD ", "") if row["type"] else ""
            data.append({
                "source": row["source"],
                "regdate": row["regdate"].strftime("%Y-%m-%d %H:%M:%S") if isinstance(row["regdate"], datetime) else row["regdate"],
                "amount": row["amount"],
                "prv_amount": row["prv_amount"],
                "next_amount": row["next_amount"],
                "vendor": row["vendor"],
                "game": row["game"],
                "etc": row["etc"],
                "type": type_cleaned
            })

        return {"status": "success", "data": data}

    except Exception as e:
        raise HTTPException(status_code=500, detail=f"데이터를 불러오는 중 오류가 발생했습니다: {str(e)}")
