# app/routes/register_user.py
from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session
from sqlalchemy.exc import SQLAlchemyError
from datetime import datetime
from app.database import get_db
from app.models.member import Member
from app.models.member_info import MembersInfo
from app.models.msg import Msg
from app.models.site_config import GameConfig
from app.models.recommender_code import RecommenderCode
from app.utils.validators import check_id_exists, check_nickname_exists, check_bank_number_exists

router = APIRouter(prefix="/api/auth", tags=["Auth"])

@router.post("/register")
async def register_user(
    join_id: str,
    join_pw: str,
    join_pw_ver: str,
    join_bank_pw: str,
    join_nick: str,
    join_phone: str,
    join_partner: str,
    join_bank: str,
    join_bank_num: str,
    join_bank_name: str,
    join_code: str = None,
    client_ip: str = None,
    db: Session = Depends(get_db)
):
    if join_pw != join_pw_ver:
        raise HTTPException(status_code=400, detail="비밀번호와 비밀번호 확인이 다릅니다.")

    if not all([join_id, join_pw, join_nick, join_phone, join_bank_pw]):
        raise HTTPException(status_code=400, detail="필수 항목을 모두 입력해주세요.")

    try:
        # 중복 데이터 확인
        if check_id_exists(join_id, db):
            raise HTTPException(status_code=400, detail="이미 사용 중인 아이디입니다.")
        if check_nickname_exists(join_nick, db):
            raise HTTPException(status_code=400, detail="이미 사용 중인 닉네임입니다.")
        if check_bank_number_exists(join_bank_num, db):
            raise HTTPException(status_code=400, detail="이미 사용 중인 계좌번호입니다.")

        # 추천인 유효성 검사
        partner = db.query(Member).filter(Member.id == join_partner, Member.active.in_([1, 5, 6])).first()
        if not partner:
            raise HTTPException(status_code=400, detail="유효하지 않은 추천인 코드입니다.")

        # 1회용 가입 코드 처리
        if join_code:
            recommender_code = db.query(RecommenderCode).filter(
                RecommenderCode.join_code == join_code,
                RecommenderCode.re_use == 0,  # Not used
                RecommenderCode.close == 0,  # Not closed
                RecommenderCode.limit_date > datetime.utcnow()  # Not expired
            ).first()

            if not recommender_code:
                raise HTTPException(status_code=400, detail="유효하지 않거나 사용된 가입 코드입니다.")

            # Mark the join code as used
            recommender_code.re_use = 1
            db.commit()

        # `Member` 데이터 삽입
        new_member = Member(
            id=join_id,
            pw=join_pw,
            pw_bank=join_bank_pw,
            nick=join_nick,
            tree_lvl=partner.tree_lvl + 1,
            lvl=7,
            recomm_id=join_partner,
            active=9  # 초기값: 승인 대기
        )
        db.add(new_member)
        db.commit()

        # `MembersInfo` 데이터 업데이트
        members_info_update = db.query(MembersInfo).filter(MembersInfo.mem_id == join_id).first()
        if not members_info_update:
            raise HTTPException(status_code=500, detail="회원정보 데이터가 생성되지 않았습니다. 트리거를 확인하세요.")

        members_info_update.phone = join_phone
        members_info_update.bank = join_bank
        members_info_update.bank_num = join_bank_num
        members_info_update.bank_name = join_bank_name
        members_info_update.join_ip = client_ip or "127.0.0.1"
        members_info_update.make_id = "user"
        members_info_update.regdate = datetime.utcnow()
        db.commit()

        # 메시지 등록
        config = db.query(GameConfig).filter(GameConfig.set_id == "admin").first()
        if config:
            msg_contents = config.g8_joinMsgContent.replace("{ID}", join_id).replace("{Nick}", join_nick)
            new_msg = Msg(in_id=join_id, contents=msg_contents, regdate=datetime.utcnow(), confim_id="admin")
            db.add(new_msg)
            db.commit()

        return {"status": "success", "message": "회원가입이 완료되었습니다."}

    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"데이터베이스 오류가 발생했습니다: {str(e)}")
    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"알 수 없는 오류가 발생했습니다: {str(e)}")
