from fastapi import APIRouter, Depends, HTTPException
from sqlalchemy.orm import Session
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.sql import func
from app.database import get_db
from app.models.request_money import RequestMoney
from app.models.total_money_log import TotalMoneyLog
from app.models.member_info import MembersInfo
from app.models.member import Member
from app.models.site_config import GameConfig
from app.models.bonus_set import BonusSet
from datetime import datetime, time
from pytz import timezone

router = APIRouter(prefix="/api/money", tags=["money"])

def format_request_money(item):
    """
    요청 데이터 형식화
    """
    result_text_map = {"1": "승인", "2": "대기", "9": "거절"}
    result_class_map = {"1": "money-ok", "2": "money-wait", "9": "money-no"}

    return {
        "rm_idx": item.rm_idx,
        "request_money": f"{item.request_money:,}",  # 숫자 포맷팅
        "regdate": item.regdate.strftime("%Y-%m-%d %H:%M:%S") if item.regdate else None,
        "result_text": result_text_map.get(item.result, ""),
        "result_class": result_class_map.get(item.result, ""),
    }

@router.get("/deposit-list")
async def get_deposit_list(
        user_id: str,  # 사용자 ID
        db: Session = Depends(get_db)
):
    """
    충전 요청 리스트 가져오기
    """
    deposits = db.query(RequestMoney).filter(
        RequestMoney.view_type == 1,
        RequestMoney.to_id == user_id,
        RequestMoney.request_type == "입금",
        RequestMoney.result.in_(["1", "2", "9"])
    ).order_by(RequestMoney.regdate.desc()).all()

    if not deposits:
        raise HTTPException(status_code=404, detail="충전 요청 데이터가 없습니다.")

    return {"status": "success", "data": [format_request_money(item) for item in deposits]}

@router.get("/withdrawal-list")
async def get_withdrawal_list(
        user_id: str,  # 사용자 ID
        db: Session = Depends(get_db)
):
    """
    환전 요청 리스트 가져오기
    """
    withdrawals = db.query(RequestMoney).filter(
        RequestMoney.view_type == 1,
        RequestMoney.to_id == user_id,
        RequestMoney.request_type == "출금",
        RequestMoney.result.in_(["1", "2", "9"])
    ).order_by(RequestMoney.regdate.desc()).all()

    if not withdrawals:
        raise HTTPException(status_code=404, detail="환전 요청 데이터가 없습니다.")

    return {"status": "success", "data": [format_request_money(item) for item in withdrawals]}

def log_total_money(
    db: Session, user_id: str, level: int, nick: str, p_lvl_1: str, p_lvl_2: str,
    p_lvl_3: str, p_lvl_4: str, p_lvl_5: str, p_lvl_6: str,
    money_type: str, request_type: str, request_money: int,
    prv_money: int, next_money: int,
    prv_point_money: int = 0, next_point_money: int = 0
):
    """
    Log the transaction in the total_money_log table, including point balances.
    """
    log_entry = TotalMoneyLog(
        id=user_id,
        lvl=level,
        nick=nick,
        p_lvl_1=p_lvl_1,
        p_lvl_2=p_lvl_2,
        p_lvl_3=p_lvl_3,
        p_lvl_4=p_lvl_4,
        p_lvl_5=p_lvl_5,
        p_lvl_6=p_lvl_6,
        money_type=money_type,
        request_type=request_type,
        request_money=request_money,
        prv_money=prv_money,
        next_money=next_money,
        prv_point_money=prv_point_money,
        next_point_money=next_point_money,
        regdate=datetime.utcnow()
    )
    db.add(log_entry)
    db.commit()

def process_deposit(
    db: Session,
    user_id: str,
    amount: int,
    money_type: str,
    bonus_type: int,
    recv_site: str = None  # 추가된 인자
):
    """
    Process deposit request and save deposit request without updating the balance immediately.
    """
    # Fetch member details
    member = db.query(Member).filter(Member.id == user_id).first()
    if not member:
        raise HTTPException(status_code=404, detail="회원 정보를 찾을 수 없습니다.")

    if member.active != 1:
        raise HTTPException(status_code=404, detail="로그 아웃 후 재신청 해주세요.")

    # Fetch member_info details
    member_info = db.query(MembersInfo).filter(MembersInfo.mem_id == user_id).first()
    if not member_info:
        raise HTTPException(status_code=404, detail="회원 은행 정보를 찾을 수 없습니다.")

    # Check for pending or in-progress requests
    existing_request = db.query(RequestMoney).filter(
        RequestMoney.request_type == "입금",
        RequestMoney.to_id == user_id,
        RequestMoney.result.in_(["2", "3"])  # 2: Pending, 3: In-progress
    ).first()

    if existing_request:
        raise HTTPException(status_code=400, detail="처리 중인 충전 내역이 있습니다. 기존 요청이 완료된 후 다시 시도해 주세요.")

    # Validate deposit amount
    if amount < 10000:
        raise HTTPException(status_code=400, detail="충전금액은 10,000원 이상 되어야 합니다.")

    # Determine admin_id and related levels
    admin_id = member.p_lvl_2

    # Count deposits and withdrawals for the day (based on KST)
    kst = timezone('Asia/Seoul')
    today_start = datetime.now(kst).replace(hour=0, minute=0, second=0, microsecond=0)
    in_count = db.query(RequestMoney).filter(
        RequestMoney.request_type == "입금",
        RequestMoney.to_id == user_id,
        RequestMoney.result == "1",
        RequestMoney.regdate >= today_start
    ).count()

    out_count = db.query(RequestMoney).filter(
        RequestMoney.request_type == "출금",
        RequestMoney.to_id == user_id,
        RequestMoney.result == "1",
        RequestMoney.regdate >= today_start
    ).count()

    site_lvl = member.site_lvl

    # Fetch bonus configuration
    bonus_set = db.query(BonusSet).filter(BonusSet.admin_id == admin_id).first()
    if not bonus_set:
        raise HTTPException(status_code=500, detail="보너스 설정을 찾을 수 없습니다.")

    # Check bonus eligibility
    bonus = bonus_type
    if in_count > 0 and out_count > bonus_set.outmoney_count_bonus and site_lvl < 5:
        bonus = 0

    # Calculate bonus amount
    bonus_amount = 0
    if bonus == 1:
        bonus_amount = amount * (0.03 if in_count > 0 else 0.05)
    elif bonus == 2:
        bonus_amount = amount * (0.05 if in_count > 0 else 0.10)
    elif bonus == 3:
        bonus_amount = amount * (0.07 if in_count > 0 else 0.15)

    # Update cumulative request money and bonus values
    lastin_b_reqmoney = amount
    lastin_b_bonus = bonus
    lastin_add_bonus = bonus_amount
    lastin_get_bonus = bonus_amount

    # Prepare account_info and account_name
    account_info = f"{member_info.bank} {member_info.bank_num} {member_info.bank_name}"
    account_name = member_info.bank_name

    # Save the deposit request
    deposit_request = RequestMoney(
        request_type="입금",
        to_id=member.id,
        to_lvl=member.lvl,
        from_id="admin",  # Default admin
        from_lvl=2,       # Default admin level
        p_lvl_1=member.p_lvl_1,
        p_lvl_2=member.p_lvl_2,
        p_lvl_3=member.p_lvl_3,
        p_lvl_4=member.p_lvl_4,
        p_lvl_5=member.p_lvl_5,
        p_lvl_6=member.p_lvl_6,
        account_info=account_info,
        account_name=account_name,
        prv_money=member.money if money_type == "money" else member.balance_money,
        request_money=amount,
        money_type=money_type,
        mem_type="member",
        phone=member_info.phone,
        ip_addr=None,  # Update if IP address is required
        result="2",  # Default to "Pending"
        regdate=datetime.now(kst),  # Use KST for the record
        site=recv_site,  # Site information
        bonus=str(bonus),
        lastin_b_reqmoney=lastin_b_reqmoney,
        lastin_b_bonus=lastin_b_bonus,
        lastin_add_bonus=lastin_add_bonus,
        lastin_get_bonus=lastin_get_bonus,
        money_type2=""
    )
    db.add(deposit_request)

    # Commit the deposit request to the database
    db.commit()

def process_withdrawal(
    db: Session, user_id: str, amount: int, money_type: str, recv_site: str = None
):
    """
    Process withdrawal request, update member balance, and log in total_money_log.
    """
    member = db.query(Member).filter(Member.id == user_id).first()
    if not member:
        raise HTTPException(status_code=404, detail="회원 정보를 찾을 수 없습니다.")

    if member.active != 1:
        raise HTTPException(status_code=404, detail="로그 아웃 후 재신청 해주세요.")

    # Check for pending or in-progress withdrawal requests
    existing_request = db.query(RequestMoney).filter(
        RequestMoney.request_type == "출금",
        RequestMoney.to_id == user_id,
        RequestMoney.result.in_(["2", "3"])  # 2: Pending, 3: In-progress
    ).first()

    if existing_request:
        raise HTTPException(status_code=400, detail="처리 중인 환전 내역이 있습니다. 기존 요청이 완료된 후 다시 시도해 주세요.")

    if amount < 10000:
        raise HTTPException(status_code=400, detail="환전금액은 10,000원 이상 되어야 합니다.")

    if amount % 10000 != 0:
        raise HTTPException(status_code=400, detail="환전금액은 10,000원 단위로 입력해야 합니다.")

    # Load GameConfig
    game_config = db.query(GameConfig).filter(GameConfig.set_id == member.p_lvl_2).first()
    if not game_config:
        raise HTTPException(status_code=500, detail="환전 설정을 찾을 수 없습니다.")

    # Fetch member_info details
    member_info = db.query(MembersInfo).filter(MembersInfo.mem_id == user_id).first()
    if not member_info:
        raise HTTPException(status_code=404, detail="회원 은행 정보를 찾을 수 없습니다.")

    # Check withdrawal time
    kst = timezone("Asia/Seoul")
    current_time = datetime.now(kst).time()
    bank_out_start = game_config.bank_out_time_str
    bank_out_end = game_config.bank_out_time_end

    if not isinstance(bank_out_start, time) or not isinstance(bank_out_end, time):
        raise HTTPException(status_code=500, detail="환전 시간 설정이 올바르지 않습니다.")

    if bank_out_start <= current_time <= bank_out_end:
        if game_config.bank_out_time_ment:
            raise HTTPException(status_code=400, detail=game_config.bank_out_time_ment)
        else:
            raise HTTPException(
                status_code=400,
                detail=f"환전 가능 시간이 아닙니다. (가능 시간: {bank_out_start.strftime('%H:%M')} ~ {bank_out_end.strftime('%H:%M')})"
            )

    # Validate maximum withdrawal per transaction
    if amount > game_config.bank_out_1order_money:
        raise HTTPException(
            status_code=400,
            detail=f"1회 최대 환전금액을 초과합니다. (제한 금액: {game_config.bank_out_1order_money:,}원)"
        )

    # Validate daily withdrawal limits
    today_start = datetime.now(kst).replace(hour=0, minute=0, second=0, microsecond=0)
    total_withdrawals_today = db.query(func.sum(RequestMoney.request_money)).filter(
        RequestMoney.request_type == "출금",
        RequestMoney.to_id == user_id,
        RequestMoney.result == "1",
        RequestMoney.regdate >= today_start
    ).scalar() or 0

    if total_withdrawals_today + amount > game_config.bank_out_1day_money:
        raise HTTPException(
            status_code=400,
            detail=f"1일 최대 환전금액을 초과합니다. (제한 금액: {game_config.bank_out_1day_money:,}원)"
        )

    daily_withdrawal_count = db.query(func.count(RequestMoney.rm_idx)).filter(
        RequestMoney.request_type == "출금",
        RequestMoney.to_id == user_id,
        RequestMoney.result == "1",
        RequestMoney.regdate >= today_start
    ).scalar() or 0

    if daily_withdrawal_count >= game_config.bank_out_1day_count:
        raise HTTPException(
            status_code=400,
            detail=f"1일 최대 환전 횟수를 초과했습니다. (제한 횟수: {game_config.bank_out_1day_count})"
        )

    # Check member's balance
    if money_type == "balance_money":
        prev_money = member.balance_money
        if prev_money < amount:
            raise HTTPException(status_code=400, detail="환전금액이 보유 금액을 초과합니다.")
        member.balance_money -= amount
        next_money = member.balance_money
    else:
        prev_money = member.money
        if prev_money < amount:
            raise HTTPException(status_code=400, detail="환전금액이 보유 금액을 초과합니다.")
        member.money -= amount
        next_money = member.money

    # Prepare account_info and account_name
    account_info = f"{member_info.bank} {member_info.bank_num} {member_info.bank_name}"
    account_name = member_info.bank_name

    # Save the withdrawal request
    withdrawal_request = RequestMoney(
        request_type="출금",
        to_id=member.id,
        to_lvl=member.lvl,
        from_id="admin",  # Default admin
        from_lvl=2,       # Default admin level
        p_lvl_1=member.p_lvl_1,
        p_lvl_2=member.p_lvl_2,
        p_lvl_3=member.p_lvl_3,
        p_lvl_4=member.p_lvl_4,
        p_lvl_5=member.p_lvl_5,
        p_lvl_6=member.p_lvl_6,
        account_info=account_info,
        account_name=account_name,
        prv_money=prev_money,
        request_money=amount,
        money_type=money_type,
        mem_type="member",
        phone=member_info.phone,
        ip_addr="127.0.0.1",  # 실제 IP로 대체 필요
        result="2",
        regdate=datetime.now(kst),  # KST 시간으로 저장
        site=recv_site
    )
    db.add(withdrawal_request)

    # Log the transaction
    log_total_money(
        db=db,
        user_id=member.id,
        level=member.lvl,
        nick=member.nick,
        p_lvl_1=member.p_lvl_1,
        p_lvl_2=member.p_lvl_2,
        p_lvl_3=member.p_lvl_3,
        p_lvl_4=member.p_lvl_4,
        p_lvl_5=member.p_lvl_5,
        p_lvl_6=member.p_lvl_6,
        money_type=money_type,
        request_type="출금",
        request_money=-amount,
        prv_money=prev_money,
        next_money=next_money,
        prv_point_money=member.point_money,  # 현재 보유 포인트
        next_point_money=member.point_money  # 포인트 변동 없음
    )

    db.commit()

@router.post("/withdraw")
async def create_withdrawal(
    user_id: str, amount: int, money_type: str = "money", site: str = None,  db: Session = Depends(get_db)
):
    """
    Create a new withdrawal request.
    """
    try:
        process_withdrawal(db=db, user_id=user_id, amount=amount, money_type=money_type, recv_site=site)
        return {"status": "success", "message": "환전 요청이 성공적으로 처리되었습니다."}
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"데이터베이스 오류가 발생했습니다: {str(e)}")
    except HTTPException as e:
        raise e
    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"알 수 없는 오류가 발생했습니다: {str(e)}")

@router.post("/deposit")
async def create_deposit(
    user_id: str,
    amount: int,
    money_type: str = "money",
    bonus_type: int = 0,
    site: str = None,  # 추가된 인자
    db: Session = Depends(get_db)
):
    """
    Create a new deposit request.
    """
    try:
        process_deposit(
            db=db,
            user_id=user_id,
            amount=amount,
            money_type=money_type,
            bonus_type=bonus_type,
            recv_site=site  # 전달
        )
        return {"status": "success", "message": "충전 요청이 성공적으로 처리되었습니다. 승인 대기 중입니다."}
    except SQLAlchemyError as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"데이터베이스 오류가 발생했습니다: {str(e)}")
    except HTTPException as e:
        raise e
    except Exception as e:
        db.rollback()
        raise HTTPException(status_code=500, detail=f"알 수 없는 오류가 발생했습니다: {str(e)}")
