# app/routes/site_config.py
from fastapi import APIRouter, Depends
from sqlalchemy.orm import Session
from datetime import datetime
from app.database import get_db
from app.models import site_config as models
from app.schemas.response_schemas import InitDataResponse

# 먼저 router 정의
router = APIRouter(
    prefix="/api/site",
    tags=["site configuration"]
)


# 닉네임 마스킹 함수 추가
def mask_nickname(nick: str) -> str:
    """닉네임을 마스킹 처리하는 함수"""
    if not nick:
        return ''
    visible_length = min(4, len(nick))
    masked_part = '*' * (12 - visible_length)
    return nick[:visible_length] + masked_part


@router.get("/init-data", response_model=InitDataResponse)
async def get_init_data(db: Session = Depends(get_db)):

    # 게임 설정 가져오기
    game_config = db.query(models.GameConfig).filter(
        models.GameConfig.set_id == 'admin'
    ).first()

    # 핫뉴스 설정 가져오기
    hotnews = db.query(models.HotNews).filter(
        models.HotNews.set_id.in_(['superadmin', 'admin'])
    ).order_by(models.HotNews.set_lvl, models.HotNews.t_index).all()

    # 출금 로그
    withdrawals = db.query(models.InOutLog).filter(
        models.InOutLog.type == 'out',
        models.InOutLog.regdate < datetime.now()
    ).order_by(models.InOutLog.regdate.desc()).limit(11).all()

    # 출금 로그 닉네임 마스킹
    withdrawals_masked = [{
        "type": w.type,
        "request": w.request,
        "regdate": w.regdate,
        "masked_nick": mask_nickname(w.nick)
    } for w in withdrawals]

    # 입금 로그
    deposits = db.query(models.InOutLog).filter(
        models.InOutLog.type == 'in',
        models.InOutLog.regdate < datetime.now()
    ).order_by(models.InOutLog.regdate.desc()).limit(11).all()

    # 입금 로그 닉네임 마스킹
    deposits_masked = [{
        "type": d.type,
        "request": d.request,
        "regdate": d.regdate,
        "masked_nick": mask_nickname(d.nick)
    } for d in deposits]

    # 프로바이더 목록
    providers = db.query(models.ProviderList).filter(
        models.ProviderList.is_live == True
    ).order_by(
        models.ProviderList.main_view_order == 0,
        models.ProviderList.main_view_order
    ).all()

    # 카지노 프로바이더
    casino_providers = db.query(models.ProviderList).filter(
        models.ProviderList.type == 'casino',
        models.ProviderList.is_live == True
    ).order_by(
        models.ProviderList.view_order == 0,
        models.ProviderList.view_order
    ).all()

    # 슬롯 프로바이더
    slot_providers = db.query(models.ProviderList).filter(
        models.ProviderList.type == 'slot',
        models.ProviderList.is_live == True
    ).order_by(
        models.ProviderList.view_order == 0,
        models.ProviderList.view_order
    ).all()

    # 상단 롤링 공지
    top_rolls = db.query(models.BoardNotice).filter(
        models.BoardNotice.board_type == 'roll',
        models.BoardNotice.hidden == False
    ).order_by(models.BoardNotice.view_cnt).all()

    # 일반 공지사항
    notices = db.query(models.BoardNotice).filter(
        models.BoardNotice.board_type == 'notice'
    ).order_by(models.BoardNotice.view_cnt).limit(5).all()

    # 공지사항 필드 필터링
    notices_filtered = [{
        "title": n.title,
        "contents": n.contents,
        "file_url": n.file_url,
        "sumfile_url": n.sumfile_url,
        "regdate": n.regdate
    } for n in notices]

    return {
        "game_config": {
            "g1_casino_use_chk": game_config.g1_casino_use_chk,
            "g1_slot_use_chk": game_config.g1_slot_use_chk,
            "g1_use_join01": game_config.g1_use_join01,
            "g1_use_join02": game_config.g1_use_join02,
            "joinSmsUse": game_config.joinSmsUse,
            "joinPartnerUse": game_config.joinPartnerUse
        },
        "hotnews": hotnews,
        "withdrawals": withdrawals_masked,
        "deposits": deposits_masked,
        "providers": providers,
        "casino_providers": casino_providers,
        "slot_providers": slot_providers,
        "top_rolls": top_rolls,
        "notices": notices_filtered
    }